"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getResourceUrl = exports.replaceResources = void 0;
const tslib_1 = require("tslib");
const typescript_1 = tslib_1.__importDefault(require("typescript"));
const constants_1 = require("../constants");
function replaceResources(shouldTransform, getTypeChecker) {
    return (context) => {
        const typeChecker = getTypeChecker();
        const resourceImportDeclarations = [];
        const moduleKind = context.getCompilerOptions().module;
        const visitNode = (node) => {
            if (typescript_1.default.isClassDeclaration(node)) {
                const decorators = typescript_1.default.visitNodes(node.decorators, (node) => typescript_1.default.isDecorator(node) ? visitDecorator(node, typeChecker, resourceImportDeclarations, moduleKind) : node);
                return typescript_1.default.updateClassDeclaration(node, decorators, node.modifiers, node.name, node.typeParameters, node.heritageClauses, node.members);
            }
            return typescript_1.default.visitEachChild(node, visitNode, context);
        };
        return (sourceFile) => {
            if (!shouldTransform(sourceFile.fileName)) {
                return sourceFile;
            }
            const updatedSourceFile = typescript_1.default.visitNode(sourceFile, visitNode);
            if (resourceImportDeclarations.length) {
                return typescript_1.default.updateSourceFileNode(updatedSourceFile, typescript_1.default.setTextRange(typescript_1.default.createNodeArray([...resourceImportDeclarations, ...updatedSourceFile.statements]), updatedSourceFile.statements));
            }
            return updatedSourceFile;
        };
    };
}
exports.replaceResources = replaceResources;
function visitDecorator(node, typeChecker, resourceImportDeclarations, moduleKind) {
    if (!isComponentDecorator(node, typeChecker)) {
        return node;
    }
    if (!typescript_1.default.isCallExpression(node.expression)) {
        return node;
    }
    const decoratorFactory = node.expression;
    const args = decoratorFactory.arguments;
    if (args.length !== 1 || !typescript_1.default.isObjectLiteralExpression(args[0])) {
        return node;
    }
    const objectExpression = args[0];
    let properties = typescript_1.default.visitNodes(objectExpression.properties, (node) => typescript_1.default.isObjectLiteralElementLike(node) ? visitComponentMetadata(node, resourceImportDeclarations, moduleKind) : node);
    const styleProperty = typescript_1.default.createPropertyAssignment(typescript_1.default.createIdentifier(constants_1.STYLES), typescript_1.default.createArrayLiteral([]));
    properties = typescript_1.default.createNodeArray([...properties, styleProperty]);
    return typescript_1.default.updateDecorator(node, typescript_1.default.updateCall(decoratorFactory, decoratorFactory.expression, decoratorFactory.typeArguments, [
        typescript_1.default.updateObjectLiteral(objectExpression, properties),
    ]));
}
function visitComponentMetadata(node, resourceImportDeclarations, moduleKind) {
    if (!typescript_1.default.isPropertyAssignment(node) || typescript_1.default.isComputedPropertyName(node.name)) {
        return node;
    }
    const name = node.name.text;
    switch (name) {
        case 'moduleId':
            return undefined;
        case constants_1.TEMPLATE_URL:
            const importName = createResourceImport(node.initializer, resourceImportDeclarations, moduleKind);
            if (!importName) {
                return node;
            }
            return typescript_1.default.updatePropertyAssignment(node, typescript_1.default.createIdentifier(constants_1.TEMPLATE), importName);
        case constants_1.STYLES:
        case constants_1.STYLE_URLS:
            if (!typescript_1.default.isArrayLiteralExpression(node.initializer)) {
                return node;
            }
            return undefined;
        default:
            return node;
    }
}
function getResourceUrl(node) {
    if (!typescript_1.default.isStringLiteral(node) && !typescript_1.default.isNoSubstitutionTemplateLiteral(node)) {
        return null;
    }
    return `${/^\.?\.\//.test(node.text) ? '' : './'}${node.text}`;
}
exports.getResourceUrl = getResourceUrl;
function isComponentDecorator(node, typeChecker) {
    if (!typescript_1.default.isDecorator(node)) {
        return false;
    }
    const origin = getDecoratorOrigin(node, typeChecker);
    return !!(origin && origin.module === '@angular/core' && origin.name === constants_1.COMPONENT);
}
function createResourceImport(node, resourceImportDeclarations, moduleKind = typescript_1.default.ModuleKind.ES2015) {
    const url = getResourceUrl(node);
    if (!url) {
        return null;
    }
    const urlLiteral = typescript_1.default.createLiteral(url);
    if (moduleKind < typescript_1.default.ModuleKind.ES2015) {
        return typescript_1.default.createCall(typescript_1.default.createIdentifier(constants_1.REQUIRE), undefined, [urlLiteral]);
    }
    else {
        const importName = typescript_1.default.createIdentifier(`__NG_CLI_RESOURCE__${resourceImportDeclarations.length}`);
        resourceImportDeclarations.push(typescript_1.default.createImportDeclaration(undefined, undefined, typescript_1.default.createImportClause(importName, undefined), urlLiteral));
        return importName;
    }
}
function getDecoratorOrigin(decorator, typeChecker) {
    if (!typescript_1.default.isCallExpression(decorator.expression)) {
        return null;
    }
    let identifier;
    let name = '';
    if (typescript_1.default.isPropertyAccessExpression(decorator.expression.expression)) {
        identifier = decorator.expression.expression.expression;
        name = decorator.expression.expression.name.text;
    }
    else if (typescript_1.default.isIdentifier(decorator.expression.expression)) {
        identifier = decorator.expression.expression;
    }
    else {
        return null;
    }
    const symbol = typeChecker.getSymbolAtLocation(identifier);
    if (symbol && symbol.declarations && symbol.declarations.length > 0) {
        const declaration = symbol.declarations[0];
        let module;
        if (typescript_1.default.isImportSpecifier(declaration)) {
            name = (declaration.propertyName || declaration.name).text;
            module = declaration.parent.parent.parent.moduleSpecifier.text;
        }
        else if (typescript_1.default.isNamespaceImport(declaration)) {
            module = declaration.parent.parent.moduleSpecifier.text;
        }
        else if (typescript_1.default.isImportClause(declaration)) {
            name = declaration.name.text;
            module = declaration.parent.moduleSpecifier.text;
        }
        else {
            return null;
        }
        return { name, module };
    }
    return null;
}
